/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.service;

import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import net.sf.parser4j.generator.entity.grammardefnode.GrammarDef;
import net.sf.parser4j.generator.entity.grammardefnode.RuleDef;
import net.sf.parser4j.generator.entity.grammardefnode.RulesDef;
import net.sf.parser4j.generator.entity.grammardefnode.RulesGroupListDef;
import net.sf.parser4j.generator.entity.grammardefnode.StatementDef;
import net.sf.parser4j.generator.service.grammardefnode.IGrammarDefVisitor;
import net.sf.parser4j.kernelgenerator.entity.grammarnode.IGrammarNode;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.parser.entity.data.NonTerminalMap;
import net.sf.parser4j.parser.entity.data.Pair;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class StatementDefToGrammarNodeAssocier implements IGrammarDefVisitor {

	private static final String START_NON_TERMINAL = "Start";

	private static final String EMPTY_STRING = "";

	private static final StatementToGrammarNode statementToGrammarNode =
	/**/StatementToGrammarNode.getInstance();

	private final transient NonTerminalMap nonTerminalMap;

	private transient Set<String> toNotExpandSet;

	private transient Map<String, Pair<StatementDef, IGrammarNode>>
	/**/statementDefAnGrammarNodeMap;

	/**
	 * @return the statementDefAnGrammarNodeMap
	 */
	public Map<String, Pair<StatementDef, IGrammarNode>>
	/**/getStatementDefAnGrammarNodeMap() {
		return statementDefAnGrammarNodeMap;
	}

	public StatementDefToGrammarNodeAssocier(final NonTerminalMap nonTerminalMap) {
		super();
		this.nonTerminalMap = nonTerminalMap;
	}

	@Override
	public void beginVisit(final GrammarDef def) {
		toNotExpandSet = new TreeSet<String>();
		statementDefAnGrammarNodeMap =
		/**/new TreeMap<String, Pair<StatementDef, IGrammarNode>>();
	}

	@Override
	public void endVisit(final GrammarDef def) {
		toNotExpandSet = null;// NOPMD free memory
	}

	@Override
	public void beginVisit(final RulesGroupListDef def) {
		// nothing to implement
	}

	@Override
	public void endVisit(final RulesGroupListDef def) {
		// nothing to implement
	}

	@Override
	public void beginVisit(final RulesDef def) {
		// nothing to implement
	}

	@Override
	public void endVisit(final RulesDef def) {
		// nothing to implement
	}

	@Override
	public void beginVisit(final RuleDef def) {
		// nothing to implement
	}

	@Override
	public void endVisit(final RuleDef def) {
		// nothing to implement
	}

	@Override
	public boolean beginVisit(final StatementDef statementDef)
			throws GrammarDefNodeVisitException {
		final boolean visit;
		final String whiteSpaceNonTerminalName = statementDef
				.getWhiteSpaceNonTerminalName();
		if (whiteSpaceNonTerminalName == null) {
			throw new IllegalStateException(
					"white space non terminal must be defined");
		}
		statementDef.setWhiteSpaceNonTerminalName(whiteSpaceNonTerminalName);
		try {

			final boolean whiteSpaceSection = !EMPTY_STRING
					.equals(whiteSpaceNonTerminalName);
			final String nonTerminalDefinedName = statementDef
					.getNonTerminalDefinedName();

			if (statementDef.isNonTerminalDefType()) {
				if (!toNotExpandSet.contains(nonTerminalDefinedName)
						&& (statementDef.getMatchClassName() != null
						/* || whiteSpaceSection */|| START_NON_TERMINAL
								.equals(nonTerminalDefinedName))) {
					changeStatementDefToConcat(statementDef);
					createGrammarNode(statementDef);
					visit = true;
				} else {
					createGrammarNode(statementDef);
					visit = true;
				}
			} else if (whiteSpaceSection && !statementDef.isTokenSon()) {
				if (!toNotExpandSet.contains(nonTerminalDefinedName)
						&& statementDef.isTerminal()
						&& !statementDef.isEmptyDefType()) {
					changeStatementDefToConcat(statementDef);
					createGrammarNode(statementDef);
					visit = true;
				} else {
					createGrammarNode(statementDef);
					visit = true;
				}
			} else {
				createGrammarNode(statementDef);
				visit = true;
			}

		} catch (GeneratorException exception) {
			throw new GrammarDefNodeVisitException(exception);
		}
		return visit;
	}

	/**
	 * change statement definition to a concatenation of its copy. match class
	 * name not copied, is null in copy
	 * 
	 * @param statementDef
	 */
	private void changeStatementDefToConcat(final StatementDef statementDef) {
		final String nonTerminalDefinedName = statementDef
				.getNonTerminalDefinedName();
		final String subNonTerminalDefinedName = "_" + nonTerminalDefinedName;
		if (statementDef.isEmptyDefType()) {
			statementDef.changeToConcat();
		} else {
			final StatementDef statementDefCopy = new StatementDef(
					statementDef, subNonTerminalDefinedName);
			statementDef.changeToConcat(statementDefCopy);
		}
		toNotExpandSet.add(subNonTerminalDefinedName);
	}

	/**
	 * create grammar node for statement definition
	 * 
	 * @param statementDef
	 * @throws GeneratorException
	 * @throws GrammarDefNodeVisitException
	 */
	private void createGrammarNode(final StatementDef statementDef)
			throws GeneratorException, GrammarDefNodeVisitException {
		final IGrammarNode grammarNode = statementToGrammarNode.create(
				statementDef, nonTerminalMap);
		addToStatementDefAnGrammarNodeMap(statementDef, grammarNode);
	}

	/**
	 * set statement definition and the grammar node for the non terminal
	 * defined by statement definition, update
	 * {@link #statementDefAnGrammarNodeMap}.
	 * 
	 * @param statementDef
	 *            the statement definition for the the non terminal defined
	 * @param grammarNode
	 *            the grammar node for the the non terminal defined
	 * @throws GrammarDefNodeVisitException
	 */
	private void addToStatementDefAnGrammarNodeMap(
			final StatementDef statementDef, final IGrammarNode grammarNode)
			throws GrammarDefNodeVisitException {
		final Pair<StatementDef, IGrammarNode> value =
		/**/new Pair<StatementDef, IGrammarNode>(statementDef, grammarNode);
		final String nonTerminalDefinedName = statementDef
				.getNonTerminalDefinedName();
		if (statementDefAnGrammarNodeMap.put(nonTerminalDefinedName, value)
		/**/!= null) {
			throw new GrammarDefNodeVisitException("duplicate definition of "
					+ nonTerminalDefinedName);
		}
	}

	@Override
	public void endVisit(final StatementDef statementDef) {
		// nothing to implement
	}
}
