/*
 * Copyright 2008 Luc Peuvrier
 * 
 * Licensed under the GNU LESSER GENERAL PUBLIC LICENSE
 * Licensed under the LGPL License, Version 3, 29 June 2007 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.gnu.org/licenses/lgpl.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.parser4j.parser.service;

public final class DebugUtil {

	private static final String NET_SF_PARSER4J = "net.sf.parser4j";

	private static final String AT = "\tat ";// NOPMD

	private final static DebugUtil INSTANCE = new DebugUtil();

	private DebugUtil() {
		super();
	}

	public static DebugUtil getInstance() {
		return INSTANCE;
	}

	/**
	 * 
	 * @return the stack trace
	 */
	public String stackTrace() {
		final Exception exception = new Exception("stack trace");// NOPMD
		return stackTrace(exception);
	}

	/**
	 * 
	 * @param throwable
	 * @return the stack trace
	 */
	public String stackTrace(final Throwable throwable) {
		final StackTraceElement[] stackTraceElements = throwable
				.getStackTrace();
		return stackTrace(stackTraceElements);
	}

	/**
	 * convert stack trace in string<br>
	 * show only element in net.sf.parser4j package<br>
	 * 
	 * @param stackTraceElements
	 * @return
	 */
	public String stackTrace(final StackTraceElement[] stackTraceElements) {
		final StringBuilder stringBuilder = new StringBuilder();
		boolean stop = false;
		for (int index = 0; !stop && index < stackTraceElements.length; index++) {
			final StackTraceElement stackTraceElement = stackTraceElements[index];
			stop = true ^ stackTraceElement.getClassName().startsWith(
					NET_SF_PARSER4J);
			if (!stop) {
				stringBuilder.append(AT);
				stringBuilder.append(stackTraceElement);
				stringBuilder.append('\n');
			}
		}
		return stringBuilder.toString();
	}
}
